/*
 * Test dataspace operations.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <ipc/cap_alloc.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "accessing.h"
#include "memory.h"
#include "memory_utils.h"
#include "pages.h"
#include "paging.h"
#include "resource_server.h"
#include "test_file_opener.h"



const unsigned int REGION_PAGES = 8;
const unsigned int MEMORY_PAGES = REGION_PAGES * 10;
const unsigned int FILE_PAGES = 20;

int main(void)
{
  /* Introduce concurrency control. */

  ipc_cap_alloc_init();

  /* Some memory plus infrastructure. */

  Memory mem(MEMORY_PAGES, page(REGION_PAGES));
  Accessing accessing;
  Paging paging;
  Pages pages(&mem);
  TestFileOpener opener(&accessing, &paging, &pages, page(FILE_PAGES));

  /* Register a server associating it with the given object. */

  ResourceServer server(&opener);
  long err = server.bind("server");

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Starting server...\n");
  server.start();
  return 0;
}
